@@ -5,6 +5,7 @@
 # --------------------------------------------------------------------------
 """Notebooklet for URL Summary."""
 from collections import Counter
+from os.path import exists
 from typing import Any, Dict, Iterable, List, Optional
 
 import dns.resolver
@@ -33,7 +34,6 @@ from ...._version import VERSION
 from ....common import (
     MsticnbDataProviderError,
     MsticnbMissingParameterError,
-    nb_data_wait,
     nb_markdown,
     set_text,
 )
@@ -51,7 +51,7 @@ _CELL_DOCS: Dict[str, Any]
 _CLS_METADATA, _CELL_DOCS = read_mod_metadata(__file__, __name__)
 
 
-# pylint: disable=too-few-public-methods
+# pylint: disable=too-few-public-methods, too-many-instance-attributes
 class URLSummaryResult(NotebookletResult):
     """URL Summary Results."""
 
@@ -73,6 +73,7 @@ class URLSummaryResult(NotebookletResult):
         self.hosts: Optional[List] = None
         self.flows: Optional[pd.DataFrame] = None
         self.flow_graph: Optional[LayoutDOM] = None
+        self.ti_results: Optional[pd.DataFrame] = None
 
 
 # pylint: disable=too-few-public-methods
@@ -156,112 +157,61 @@ class URLSummary(Notebooklet):
         result = URLSummaryResult(
             notebooklet=self, description=self.metadata.description, timespan=timespan
         )
+        if not self._last_result:
+            self._last_result = result
 
-        url = value.strip().lower()
-        _, domain, tld = tldextract.extract(url)
+        self.url = value.strip().lower()
+        _, domain, tld = tldextract.extract(self.url)
         domain = f"{domain.lower()}.{tld.lower()}"
         domain_validator = DomainValidator()
         validated = domain_validator.validate_tld(domain)
 
         result.summary = pd.DataFrame(
-            {"URL": [url], "Domain": [domain], "Validated TLD": [validated]}
+            {"URL": [self.url], "Domain": [domain], "Validated TLD": [validated]}
         )
-        if not self.silent:
-            nb_markdown(f"Summary of {url}:")
-            display(result.summary)
 
         if "ti" in self.options:
             if "tilookup" in self.data_providers.providers:
                 ti_prov = self.data_providers.providers["tilookup"]
             else:
                 raise MsticnbDataProviderError("No TI providers avaliable")
-            nb_data_wait("Threat Intelligence Results")
             ti_results, ti_results_merged = get_ti_results(
                 ti_prov, result.summary, "URL"
             )
             if isinstance(ti_results, pd.DataFrame) and not ti_results.empty:
-                result.summary = ti_results_merged
-            if not self.silent:
-                nb_markdown(f"Threat Intelligence Results for {url}.")
-                display(ti_results_merged)
+                result.ti_results = ti_results_merged
 
         if "whois" in self.options:
             result.domain_record = _domain_whois_record(
                 domain, self.data_providers.providers["tilookup"]
             )
-            if (
-                not self.silent
-                and isinstance(result, pd.DataFrame)
-                and not result.domain_record.empty  # type: ignore
-            ):
-                nb_markdown(f"WhoIs Results for {url}.")
-                display(
-                    result.domain_record.T.style.applymap(  # type: ignore
-                        color_domain_record_cells,
-                        subset=pd.IndexSlice[["Page Rank", "Domain Name Entropy"], 0],
-                    )
-                )
 
         if "cert" in self.options:
-            result.cert_details = _get_tls_cert_details(url, domain_validator)
-            if not self.silent:
-                if (
-                    isinstance(result.cert_details, pd.DataFrame)
-                    and not result.cert_details.empty
-                ):
-                    nb_markdown(f"TLS Certificate Details for {url}.")
-                    display(result.cert_details)
-                else:
-                    print("No TLS certificate found.")
+            result.cert_details = _get_tls_cert_details(self.url, domain_validator)
 
         if "ip_record" in self.options:
+            result.ip_record = None
             result.ip_record = _get_ip_record(
                 domain, domain_validator, self.data_providers.providers["tilookup"]
             )
-            if not self.silent:
-                if (
-                    isinstance(result.ip_record, pd.DataFrame)
-                    and not result.ip_record.empty
-                ):
-                    nb_markdown(f"IP Address Details for {url}.")
-                    display(result.ip_record.T)
-                else:
-                    print("No current IP found.")
 
         if "screenshot" in self.options:
-            image_data = screenshot(url)
+            image_data = screenshot(self.url)
             with open("screenshot.png", "wb") as screenshot_file:
                 screenshot_file.write(image_data.content)
-            if not self.silent:
-                nb_markdown(f"Screenshot of {url}")
-                display(Image(filename="screenshot.png"))
 
         if "alerts" in self.options:
             alerts = self.query_provider.SecurityAlert.list_alerts(timespan)
             result.related_alerts = alerts[
-                alerts["Entities"].str.contains(url, case=False)
+                alerts["Entities"].str.contains(self.url, case=False)
             ]
-            if (
-                not self.silent
-                and isinstance(result, pd.DataFrame)
-                and not result.related_alerts.empty  # type: ignore
-            ):
-                nb_markdown(f"Alerts related to {url}")
-                display(result.related_alerts)
 
         if "bookmarks" in self.options:
             result.bookmarks = (
                 self.query_provider.AzureSentinel.list_bookmarks_for_entity(
-                    url=url, start=timespan.start, end=timespan.end
+                    url=self.url, start=timespan.start, end=timespan.end
                 )
             )
-            if (
-                not self.silent
-                and isinstance(result, pd.DataFrame)
-                and not result.bookmarks.empty  # type: ignore
-            ):
-                nb_markdown(f"Bookmarks related to {url}")
-                display(result.bookmarks)
 
         if "dns" in self.options:
             result.dns_results = (
@@ -269,33 +219,27 @@ class URLSummary(Notebooklet):
                     domain=domain, start=timespan.start, end=timespan.end
                 )
             )
-            if not self.silent and not result.dns_results.empty:  # type: ignore
-                nb_markdown(f"DNS events related to {url}")
-                display(result.dns_results)
 
         if "hosts" in self.options:
             syslog_hosts = self.query_provider.LinuxSyslog.all_syslog(
-                add_query_items=f"| where SyslogMessage has '{url}'",
+                add_query_items=f"| where SyslogMessage has '{self.url}'",
                 start=timespan.start,
                 end=timespan.end,
             )["Computer"].unique()
             mde_hosts = self.query_provider.MDE.host_connections(
                 time_column="TimeGenerated",
                 host_name="",
-                add_query_items=f"| where RemoteUrl has '{url}'",
+                add_query_items=f"| where RemoteUrl has '{self.url}'",
                 start=timespan.start,
                 end=timespan.end,
             )["DeviceName"].unique()
             windows_hosts = self.query_provider.WindowsSecurity.list_events(
-                add_query_items=f"| where CommandLine has '{url}'",
+                add_query_items=f"| where CommandLine has '{self.url}'",
                 start=timespan.start,
                 end=timespan.end,
             )["Computer"].unique()
             all_hosts = list(syslog_hosts) + list(mde_hosts) + list(windows_hosts)
             result.hosts = all_hosts
-            if not self.silent:
-                nb_markdown(f"Hosts connecting to {url}")
-                display(result.hosts)
 
         if "flows" in self.options:
             result.flows = self.query_provider.Network.network_connections_to_url(
@@ -310,16 +254,111 @@ class URLSummary(Notebooklet):
             result.flow_graph = display_timeline_values(
                 flow_graph_data,
                 value_col="sum_SentBytes",
-                title=f"Network traffic volume to {url}",
+                title=f"Network traffic volume to {self.url}",
             )
-            if not self.silent:
-                display(result.flow_graph)
-                nb_markdown(f"Network connections to {url}")
-                display(result.flows)
 
         self._last_result = result
+
+        if not self.silent:
+            self._display_results()
+
         return self._last_result
 
+    @set_text(docs=_CELL_DOCS, key="display_summary")
+    def _display_summary(self):
+        """Display URL summary."""
+        if self.check_valid_result_data("summary", silent=True):
+            display(self._last_result.summary)
+
+    @set_text(docs=_CELL_DOCS, key="show_ti_details")
+    def _display_ti_data(self):
+        """Display TI results."""
+        if self.check_valid_result_data("ti_results", silent=True):
+            display(self._last_result.ti_results)
+        else:
+            nb_markdown(f"No TI results found for {self.url}")
+
+    @set_text(docs=_CELL_DOCS, key="show_domain_record")
+    def _display_domain_record(self):
+        """Display Domain Record."""
+        if self.check_valid_result_data("domain_record", silent=True):
+            display(
+                self._last_result.domain_record.T.style.applymap(  # type: ignore
+                    color_domain_record_cells,
+                    subset=pd.IndexSlice[["Page Rank", "Domain Name Entropy"], 0],
+                )
+            )
+
+    @set_text(docs=_CELL_DOCS, key="show_TLS_cert")
+    def _display_cert_details(self):
+        """Display TLS Certificate details."""
+        if self.check_valid_result_data("cert_details", silent=True):
+            display(self._last_result.cert_details)
+        else:
+            nb_markdown(f"No TLS certificate found for {self.url}.")
+
+    @set_text(docs=_CELL_DOCS, key="show_IP_record")
+    def _display_ip_record(self):
+        """Display IP record."""
+        if self.check_valid_result_data("ip_record", silent=True):
+            display(self._last_result.ip_record.T)
+        else:
+            nb_markdown(f"No current IP found for {self.url}.")
+
+    @set_text(docs=_CELL_DOCS, key="show_screenshot")
+    def _display_screenshot(self):
+        """Display ULR screenshot."""
+        if exists("screenshot.png"):
+            nb_markdown(f"Screenshot of {self.url}")
+            display(Image(filename="screenshot.png"))
+
+    @set_text(docs=_CELL_DOCS, key="show_related_alerts")
+    def _display_related_alerts(self):
+        """Display related alerts in table."""
+        if self.check_valid_result_data("related_alerts", silent=True):
+            display(self._last_result.related_alerts)
+        else:
+            nb_markdown(f"No Alerts related to {self.url}")
+
+    @set_text(docs=_CELL_DOCS, key="show_bookmarks")
+    def _display_bookmarks(self):
+        """Display bookmarks related to URL."""
+        if self.check_valid_result_data("bookmarks", silent=True):
+            display(self._last_result.bookmarks)
+        else:
+            nb_markdown(f"No Bookmarks related to {self.url}")
+
+    @set_text(docs=_CELL_DOCS, key="show_dns_results")
+    def _display_dns_results(self):
+        """Display DNS resolutions for URL."""
+        if self.check_valid_result_data("dns_results", silent=True):
+            nb_markdown(f"DNS events related to {self.url}", "bold")
+            display(self._last_result.dns_results)
+        else:
+            nb_markdown(f"No DNS resolutions found for {self.url}")
+
+    @set_text(docs=_CELL_DOCS, key="show_hosts")
+    def _display_hosts(self):
+        """Display list of hosts connecting to URL."""
+        if (
+            self.check_valid_result_data("hosts", silent=True)
+            and self._last_result.hosts
+        ):
+            nb_markdown(f"Hosts connecting to {self.url}", "bold")
+            display(self._last_result.hosts)
+        else:
+            nb_markdown(f"No hosts found connecting to {self.url}")
+
+    @set_text(docs=_CELL_DOCS, key="show_flows")
+    def _display_flows(self):
+        """Display network flow data for URL."""
+        if self.check_valid_result_data("flow_graph", silent=True):
+            display(self._last_result.flow_graph)
+            nb_markdown(f"Network connections to {self.url}", "bold")
+            display(self._last_result.flows)
+        else:
+            nb_markdown(f"No flow data found for {self.url}")
+
     @set_text(docs=_CELL_DOCS, key="browse_alerts")
     def browse_alerts(self) -> nbwidgets.SelectAlert:
         """Return alert browser/viewer."""
@@ -335,6 +374,20 @@ class URLSummary(Notebooklet):
             md("Cannot plot timeline with 0 or 1 event.")
         return None
 
+    def _display_results(self):
+        """Display all the notebooklet results."""
+        self._display_summary()
+        self._display_domain_record()
+        self._display_ip_record()
+        self._display_cert_details()
+        self._display_ti_data()
+        self._display_screenshot()
+        self._display_related_alerts()
+        self._display_bookmarks()
+        self._display_dns_results()
+        self._display_hosts()
+        self._display_flows()
+
 
 def entropy(data):
     """Calculate Entropy of a String."""
@@ -370,7 +423,6 @@ def _show_alert_timeline(related_alerts):
     return None
 
 
-@set_text(docs=_CELL_DOCS, key="show_domain_record")
 def _domain_whois_record(domain, ti_prov):
     """Build a Domain Whois Record."""
     dom_record = pd.DataFrame()
@@ -396,27 +448,35 @@ def _domain_whois_record(domain, ti_prov):
         )
         ns_domains = []
 
-        # Identity domains populatirty with Open Page Rank
-        page_rank = ti_prov.result_to_df(
-            ti_prov.lookup_ioc(observable=domain, providers=["OPR"])
-        )
-        if page_rank["RawResult"][0]:
-            page_rank_score = page_rank["RawResult"][0]["response"][0][
-                "page_rank_integer"
-            ]
+        # Identity domains popularity with Open Page Rank
+        if "OPR" in ti_prov.loaded_providers:
+            page_rank = ti_prov.result_to_df(
+                ti_prov.lookup_ioc(domain, providers=["OPR"])
+            )
+            if page_rank["RawResult"][0]:
+                page_rank_score = page_rank["RawResult"][0]["response"][0][
+                    "page_rank_integer"
+                ]
+            else:
+                page_rank_score = 0
+            dom_record["Page Rank"] = [page_rank_score]
         else:
-            page_rank_score = 0
-        dom_record["Page Rank"] = [page_rank_score]
+            nb_markdown("OPR TI provider needed to calculate Page Rank score.")
+            dom_record["Page Rank"] = ["Not known - OPR provider needed"]
 
         # Get a list of subdomains for the domain
-        url_ti = ti_prov.result_to_df(
-            ti_prov.lookup_ioc(observable=domain, providers=["VirusTotal"])
-        )
-        try:
-            sub_doms = url_ti["RawResult"][0]["subdomains"]
-        except (TypeError, KeyError):
-            sub_doms = "None found"
-        dom_record["Sub Domains"] = [sub_doms]
+        if "VirusTotal" in ti_prov.loaded_providers:
+            url_ti = ti_prov.result_to_df(
+                ti_prov.lookup_ioc(domain, providers=["VirusTotal"])
+            )
+            try:
+                sub_doms = url_ti["RawResult"][0]["subdomains"]
+            except (TypeError, KeyError):
+                sub_doms = "None found"
+            dom_record["Sub Domains"] = [sub_doms]
+        else:
+            nb_markdown("VT TI provider needed to get sub-domains.")
+            dom_record["Page Rank"] = ["Not known - OPR provider needed"]
 
         # Work out domain entropy to identity possible DGA
         dom_ent = entropy(domain)
@@ -431,7 +491,6 @@ def _domain_whois_record(domain, ti_prov):
     return dom_record
 
 
-@set_text(docs=_CELL_DOCS, key="show_TLS_cert")
 def _get_tls_cert_details(url, domain_validator):
     """Get details of a TLS certificate used by a domain."""
     result, x509 = domain_validator.in_abuse_list(url)
@@ -449,9 +508,8 @@ def _get_tls_cert_details(url, domain_validator):
     return cert_df
 
 
-@set_text(docs=_CELL_DOCS, key="show_IP_record")
 def _get_ip_record(domain, domain_validator, ti_prov):
-    """Get IP addresses assoicated with a domain."""
+    """Get IP addresses associated with a domain."""
     ip_record = None
     if domain_validator.is_resolvable(domain) is True:
         try:
@@ -469,7 +527,6 @@ def _get_ip_record(domain, domain_validator, ti_prov):
                 "Creation Date": [ip_whois_result.get("creation_date", None)],
             }
         )
-
     if isinstance(ip_record, pd.DataFrame) and not ip_record.empty:
         ip_record = _process_tor_ip_record(ip_record, ti_prov)
         ip_record = _process_previous_resolutions(ip_record, ti_prov)
@@ -480,8 +537,9 @@ def _process_tor_ip_record(ip_record, ti_prov):
     """See if IP record contains Tor IP."""
     tor = None
     if "Tor" in ti_prov.loaded_providers:
+        print(ti_prov.loaded_providers)
         tor = ti_prov.result_to_df(
-            ti_prov.lookup_ioc(observable=ip_record["IP Address"][0], providers=["Tor"])
+            ti_prov.lookup_ioc(ip_record["IP Address"][0], providers=["Tor"])
         )
     if tor is None or tor["Details"][0] == "Not found.":
         ip_record["Tor Node?"] = "No"
@@ -492,15 +550,18 @@ def _process_tor_ip_record(ip_record, ti_prov):
 
 def _process_previous_resolutions(ip_record, ti_prov):
     """Get previous resolutions for IP in ip_record."""
-    ip_ti_results = ti_prov.result_to_df(
-        ti_prov.lookup_ioc(
-            observable=ip_record["IP Address"][0], providers=["VirusTotal"]
+    if "VirusTotal" in ti_prov.loaded_providers:
+        ip_ti_results = ti_prov.result_to_df(
+            ti_prov.lookup_ioc(ip_record["IP Address"][0], providers=["VirusTotal"])
+        )
+        try:
+            last_10 = ip_ti_results.T["VirusTotal"]["RawResult"]["resolutions"][:10]
+            prev_domains = [record["hostname"] for record in last_10]
+        except TypeError:
+            prev_domains = None
+    else:
+        prev_domains = (
+            "Unknown - VirusTotal provider required for previous resolution details."
         )
-    )
-    try:
-        last_10 = ip_ti_results.T["VirusTotal"]["RawResult"]["resolutions"][:10]
-        prev_domains = [record["hostname"] for record in last_10]
-    except TypeError:
-        prev_domains = None
     ip_record["Last 10 resolutions"] = [prev_domains]
     return ip_record
